/*
 * @(#)ManhattanPoint.java  1.0  17 November 2002
 * <P> A class to represent a point in some planar coordinate space. </P>
 */
public class ManhattanPoint {

	/*------------------------------------------------------------------------- 
	 Data Definition: 
    -------------------------------------------------------------------------*/
	
	/* The x coordinate. */
	int x;
	
	/* The y coordinate. */
	int y;
		
	/*------------------------------------------------------------------------- 
	 The constructor
	 ---------------
	 Purpose: to create an object in the class ManhattanPoint and initialize
	 		  all member data.
	 
	 Examples:
	 		ManhattanPoint myManhattanPoint    = new ManhattanPoint(16, 7);
    		ManhattanPoint otherManhattanPoint = new ManhattanPoint(5, 13);
    -------------------------------------------------------------------------*/
	ManhattanPoint(int some_x, int some_y){
		this.x = some_x;
		this.y = some_y;
	}
	
	/*------------------------------------------------------------------------- 
	 Methods: 
    -------------------------------------------------------------------------*/

	/*------------------------------------------------------------------------- 
	 Purpose: 	to find the distance to the city center at (0, 0) 
	 
	 Examples: 
		        myManhattanPoint = new ManhattanPoint(6, 17);
		        otherManhattanPoint = new ManhattanPoint(5, 13);
	 			
	 			expected( 23 );
	 			actual  ( myManhattanPoint.distanceTo0() );
	 				 				     
	 			expected( 18 );
	 			actual  ( otherManhattanPoint.distanceTo0() );
		  	       
	 Template: 	int <- distanceTo0(){
					return ... this.x ... this.y ... ;
			  	} 
    -------------------------------------------------------------------------*/
	int distanceTo0(){
		return this.x + this.y;
	}
	

	/*------------------------------------------------------------------------- 
	 Contract: 	boolean <- isWithin(int someBlocks)
	 
	 Purpose: 	to determine whether this point 
	 			is within the given number of blocks from the city center
	  
	 Examples:
		        myManhattanPoint = new ManhattanPoint(6, 17);
		        otherManhattanPoint = new ManhattanPoint(5, 13);
	 			
	 			expected( false );
	 			actual  ( myManhattanPoint.isWithin(20) );
	 				 				     
	 			expected( true );
	 			actual  ( otherManhattanPoint.isWithin(20) );
		  	       
		        myManhattanPoint.distanceTo0(): 23 
		        otherManhattanPoint.distanceTo0(): 18
		        		  	       
	 Template:	boolean <- isWithin(someBlocks){
	 				... this.x ... this.y ... someBlocks ...
	 			}
    -------------------------------------------------------------------------*/
	boolean isWithin(int someBlocks){
		return (this.distanceTo0() < someBlocks);		
	}
	
	
	/*------------------------------------------------------------------------- 
	 Contract: boolean <- isCloserThan(ManhattanPoint somePoint)
	 
	 Purpose: 	to determine whether this point is closer to the city center
	 		  	that a given ManhattanPoint
	  
	 Examples:
		        myManhattanPoint = new ManhattanPoint(6, 17);
		        otherManhattanPoint = new ManhattanPoint(5, 13);
	 			
		        myManhattanPoint.distanceTo0(): 23
		        otherManhattanPoint.distanceTo0(): 18
		        
		        true  <- myManhattanPoint.isCloserThan(otherManhattanPoint);
		        false <- otherManhattanPoint.isCloserThan(myManhattanPoint);
	 		
	 Template: 	boolean <- isCloserThan(somePoint){
	 				... this.x ... this.y ... somePoint.x ... somePoint.y ...
	 			}
    -------------------------------------------------------------------------*/
	boolean isCloserThan(ManhattanPoint somePoint){
		return ( this.distanceTo0() < somePoint.distanceTo0() );		
	}
	
	
	/*------------------------------------------------------------------------- 
	 Contract: ManhattanPoint <- getCloserPoint(ManhattanPoint somePoint)
	 
	 Purpose: to return the point that is closer to the city center
	 		  choosing between this and a given ManhattanPoint
	  
		        myManhattanPoint = new ManhattanPoint(6, 17);
		        otherManhattanPoint = new ManhattanPoint(5, 13);
	 			
		        myManhattanPoint.distanceTo0(): 23
		        otherManhattanPoint.distanceTo0(): 18
	 				     
	 	myManhattanPoint <- myManhattanPoint.getCloserPoint(otherManhattanPoint);
	 	myManhattanPoint <- otherManhattanPoint.getCloserPoint(myManhattanPoint);
	 		
	 Template: 	ManhattanPoint <- getCloserPoint(somePoint){
	 				... this.x ... this.y ... somePoint.x ... somePoint.y ...
	 			}
    -------------------------------------------------------------------------*/
	ManhattanPoint getCloserPoint(ManhattanPoint somePoint){

		if ( this.distanceTo0() < somePoint.distanceTo0() )
			return this;
		else
			return somePoint;		
	}
	
	
	/*------------------------------------------------------------------------- 
	 Helper method to print the object member data.
    -------------------------------------------------------------------------*/
	public String toString(){
		return("new " + getClass().getName() + "(" + 
					x  + ", " + 
					y +  ")");
	}
}
